/*
 *  5C Array Builder
 *
 *	Builds the Array
 *	
 *  Created by James Fraser on 8/10/07.
 *  
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <time.h>
#include "OligoTools.c"
#include "ArrayTools.c"

//Prints the help function
int help() {
	
	printf("5CArrayBuilder V1.2\n\n");
	printf("Incorrect commandline entry.\n");
	printf("Options:\n");
	printf("\t-f FILENAME\tLocation of the file with forward primers to be imported, followed by a range of fragment numbers.\n");
	printf("\t-r FILENAME\tLocation of the file with reverse primers to be imported, followed by a range of fragment numbers.\n");
	printf("\t\t\tIn both of the above, a range of primers can consist of one or more ranges (denoted by dashes) or single primers, seperated by dashes. ");
	printf("There should be no spaces in the statement. See below for example.\n");
	printf("\t-o FILENAME\tLocation of file output\n");
	printf("\t-pf PREFIX \tPrefix attached to forward primers\n");
	printf("\t-pr PREFIX \tPrefix attached to forward primers\n");
	printf("\t-i\t Includes F / R primers with the same fragment number (Use this for intra region arrays)\n");
	printf("------------------------\nEXAMPLE USAGE:\n");
	printf("\"./5CAB -f ~/HoxA.txt 35-95,100,101,102-109 -r ~/HoxD.txt 21-45,59-80 -o ~/HoxA_HoxDarray.txt -pf HoxA -pr HoxD -i\"\n");
	
	return EXIT_SUCCESS;

}

int main(int argc, char *argv[]) {

	char r_filename[100], f_filename[100], o_filename[100], r_site[200], f_site[200], forward_prefix[30], reverse_prefix[30];
	int i=0,l=0, sameNumber = 0, n=0, x=0, m=0, index1, index2, primer_number=1000, forward_length, reverse_length, forward_max, reverse_max;
	int reverse_numbers[500], forward_numbers[500], site_lengths[8] = {15,18,19,20,21,22,23,24};
	FILE *f_file, *r_file, *output;
	
	//Default prefix values
	strcpy(forward_prefix, "Forward");
	strcpy(reverse_prefix, "Reverse");
			
	//Ensures help will be reached if no arguments are present
	if (argc <= 1) {
		
		help();
		return EXIT_SUCCESS;
		
	}
	
	/* Command Input */
	for ( i = 1; i < argc; i++) {
		
		if ( strcmp("-f", argv[i]) == 0 ) {		//Reads in file used for forward primers
			
			i++;
			strcpy(f_filename, argv[i]);
			f_file = fopen(f_filename, "r");
			
			if (f_file == NULL) {
			
				printf("\nError: Can't open the file \"%s\"!.\n", argv[i]);
				return EXIT_FAILURE;
			
			}
			
			i++;
			
			list(argv[i], forward_numbers);

		} else if ( strcmp("-r", argv[i]) == 0) { //Reads in file used for reverse primers
		
			i++;
			strcpy(r_filename, argv[i]);
			r_file = fopen(r_filename, "r");
			
			if (r_file == NULL) {
			
				printf("\nError: Can't open the file \"%s\"!.\n", argv[i]);
				return EXIT_FAILURE;
			
			}
			
			i++;
			
			list(argv[i], reverse_numbers);
			

		} else if (strcmp("-o", argv[i]) == 0) { //Creates output file
		
			i++;
			strcpy(o_filename, argv[i]);
			output = fopen(o_filename, "w");
			
			if (output == NULL) {
			
				printf("\nError: Can't open the file \"%s\" to write to!.\n", argv[i]);
				return EXIT_FAILURE;
			
			}			
					
		} else if (strcmp("-i", argv[i]) == 0) {//Flag to tell the code to include the same numbered primers. Used in intra-region arrays
		
			sameNumber = 1; 			
					
		} else if (strcmp("-pf", argv[i]) == 0) {//Used to generate a prefix describing the forward region
			
			i++;
			strcpy(forward_prefix, argv[i]);			
			
		} else if (strcmp("-pr", argv[i]) == 0) {//Used to generate a prefix describing the reverse region
			
			i++;
			strcpy(reverse_prefix, argv[i]);
			
		} else {
		
			help(); //occurs if anything other than the specified format appears, so the program exits
		
		}
		
	}
	
	//End of commandline reading
	
	
	//Start of algorithm
	
	printf("Building Array with %s and %s...\n",f_filename, r_filename);

	//Generates arrays to hold each of the sets of primers
	char **forward_primers = (char **)calloc( primer_number, sizeof(char) );
	for (i = 0; i < primer_number; i++) {
		forward_primers[i] = calloc(50, sizeof(char));
	}
	
	char **reverse_primers = (char **)calloc( primer_number, sizeof(char) );
	for (i = 0; i < primer_number; i++) {
		reverse_primers[i] = calloc(50, sizeof(char));
	}
		
	//Imports data from the files into the arrays
	forward_max = process_input(f_file, forward_primers, 1);
	reverse_max = process_input(r_file, reverse_primers, -1);
			
	//Main loop which goes through all the forward primers
	for (i = 0; i < forward_max; i++) {
					
		//Decides if we have the right forward primer and can move on
		if ( forward_numbers[n] == i) {
			
			//Loops through all the reverse primers
			for (x =0; x < reverse_max; x++) {
				
				//Decides if we have the right reverse primer and can move on
				if (reverse_numbers[m] == x) {
					
							
					if (i==x && sameNumber == 0) {
						
						//Do nothing, so that we skip any matching forward and reverse primers
						
					} else {
						
						//We have a set of primers that can now be matched together.
						//i = forward number
						//x = reverse number
						
						forward_length = strlen(forward_primers[i-1]);
						reverse_length = strlen(reverse_primers[x-1]);
						
						//Generates the appropriate half site primers
						for (l = 0; l < 8; l++)
						{
							
							//Forward primer
							index1 = (forward_length - site_lengths[l]);
							if (index1 < 0)
							{
								
								index1 = 0; //ensures halfsite isnt larger than the primer
								
							}
														
							//reverse primer
							index2 = (site_lengths[l]-1);
							if (index2 > reverse_length)
							{
								
								index2 = reverse_length; //ensures halfsite isnt larger than the primer
								
							}
							
							
							//extract forward primer
							extract(forward_primers[i-1], index1 , forward_length, f_site);
							
							//extract reverse primer
							extract(reverse_primers[x-1], 0 , index2, r_site);
							
							fprintf(output,"%s_F_%d-%s_R_%d-%d\t%s%s\n", forward_prefix, i,reverse_prefix,x, site_lengths[l],f_site, r_site);
							
						}
						
						//printf("Forward: %d \tReverse: %d\n", i, x);
						
					}					
					
					m++;
									
				}
								
			}
			
			m = 0;		
			n++;	
			
		}	
					
	}
	
	return EXIT_SUCCESS;

}
