/*
 *  5C Primer Design
 *  
 *  Used to generate primers for use with high-throughput 5C experiments
 *
 *  Created by James Fraser on 8/10/07.
 *  
 *
 TODO:
  
 
 
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>

#import "OligoTools.c"

double version = 2.22;

void help(){
	
	printf("5CP V%f\n\n", version);
	printf("Not enough options were specified!:\n");
	printf("Options:\n");
	printf("\t-f FILENAME\tLocation of the file to be imported\n");
	printf("\t-e CODE\t\t6-letter DNA code representing the cut sequence\n");
	printf("\t-t NUMBER\tDesired Tm for Oligos\n");
	printf("\t-c NUMBER\tDesired number of cycles\n");
	printf("\t-o FILENAME\tLocation of file output\n");
	printf("\t-r PATH\tLocation of RepeatMasker Program. Without this option specified, Repeatmasker will not be run\n");
	printf("\t-organism NAME\tThe name of the organism for use in RepeatMasker (Default:human)\n");
	printf("\nEXAMPLE:\n\t");
	printf("./5CPrimer -f ~/Desktop/input.txt -e ACATGT -t 72 -c 72 -o ~/Desktop/output.txt -r ~/bin/RepeatMasker/RepeatMasker -organism human");
	
}

int main(int argc, char *argv[]) {
	
	if (argc == 1) {
		
		help();
		return EXIT_SUCCESS;
		
	}
	

	/* Variable initiallization */
	int buf = 100000000, n, i, cut[500], x, z;
	char output[200], filename[200], enzyme[30], temp[100], header[200], F[30], R[30], code_plus[50], noncoding[200], mask[200], ncmask[200], organism[300], masker[500];
	char *conc = calloc( 50, sizeof(int) );
	double T, perc_repeat;
	FILE *output_file;
	char *masked;
	
	time_t t1, t2; //Used to show the time taken
	(void) time(&t1); // Time 1
	
	int MAX_TM = 76, MAX_CYCLES = 72; //Default values
	int RepMask = 0; //Default value, repeatmasker is turned off.
		
	strcpy(F, "TAATACGACTCACTATAGCC"); // T7
	strcpy(R, "TCCCTTTAGTGAGGGTTAATA"); // T3c
	strcpy(organism, "human");//Organism for RepeatMasker

	
	/* Command Input */
	for ( i = 1; i < argc; i++) {
		
		if ( strcmp("-f", argv[i]) == 0 ) {
			
			i++;
			strcpy(filename, argv[i]);
								
		} else if ( strcmp("-e", argv[i]) == 0) {
		
			i++;
			strcpy(enzyme, argv[i]);
			
			for (z = 0; z < strlen(enzyme); z++) { // Checks for invalid characters
			
				//Capitalizes any lowercase letters
				if (enzyme[z] == 'a') {
				
					enzyme[z] = 'A';
				
				} else if (enzyme[z] == 'c') {
				
					enzyme[z] = 'C';
				
				} else if (enzyme[z] == 'g') {
				
					enzyme[z] = 'G';
				
				} else if (enzyme[z] == 't') {
				
					enzyme[z] = 'T';
				
				} 
				
				//Exits on any invalid cut sequences.			
				if (enzyme[z] != 'A' && enzyme[z] != 'C' && enzyme[z] != 'G' && enzyme[z] != 'T') {
				
					printf("Invalid enzyme cut sequence!");
					return EXIT_FAILURE;
									
				}			
			
			}						
												
			if (strlen(enzyme)%2 != 0) //True for even numbered cut sites
			{
			
				printf("Invalid enzyme cut sequence!");
				return EXIT_FAILURE;
			
			}

		} else if ( strcmp("-t", argv[i]) == 0) {
		
			i++;
			MAX_TM = atoi(argv[i]);
			if (MAX_TM == 0) {printf("Invalid Number for Tm");}
			
		} else if ( strcmp("-c", argv[i]) == 0) {
		
			i++;
			MAX_CYCLES = atoi(argv[i]);
			if (MAX_CYCLES == 0) {printf("Invalid Number for cycles");}
		
		} else if ( strcmp("-o", argv[i]) == 0) {
		
			i++;
			strcpy(output, argv[i]);
			//Should check for directory

		} else if ( strcmp("-r", argv[i]) == 0) {
			
			i++;
			
			strcpy(masker, argv[i]);
			RepMask = 1;
			//Turns on RepeatMasker
			
		} else if ( strcmp("-organism", argv[i]) == 0) {
			
			i++;
			strcpy(organism, argv[i]);
			//Switches organism to something else in RepeatMasker
			
		} else {
			
			help();
			return EXIT_SUCCESS;
			
		}
			
	}
	/* End command input */
	
				
	/* Allocating memory for array */
	char *code = calloc( buf, sizeof(char) );
	n = importDNA(filename, code, header);
	
	//Runs repeatMasker, extracts DNA. Runs only if turned on.
	if (RepMask == 1) {
		
		masked = calloc( buf, sizeof(char) );		
		if (RepeatMasker(filename, masked, organism, masker) == EXIT_FAILURE)
		{
		
			printf("Repeatmasker could not be started. Proceeding without...\n\n");
			RepMask = 0;
			
		}
				
	}
	
	// Identifies cut sites
	cutter(code, enzyme, n, cut);
		
	/* Opens the output file, and returns an error message if no file exists */
	if ( (output_file = fopen(output, "w")) == NULL ) {
	
		printf("Error Creating File: %s\n", output);
		return EXIT_FAILURE;
	
	}
		
	/* Header information */
	fprintf(output_file, "5CPrimer Version %f 5C Primers using the following parameters:\n", version);
	fprintf(output_file, "Input: %s | Tm: %d | Cycles: %d | Enzyme: %s | Output: %s\nHeader: %s\n\n", filename, MAX_TM, MAX_CYCLES, enzyme, output, header);
	fprintf(output_file, "Fragment_Number\tPosition\tTm\tCycles\tGenomic_Size\tForward_Genomic\tForward_5CPrimer\tReverse_Genomic\tReverse_5CPrimer\t");
	
	if (RepMask == 1) {
	
		//print RepeatMasker Legend
		fprintf(output_file, "Percent_Repeat\tForward_Masked\tReverse_Masked");
		
	}
	
	fprintf(output_file, "\n");
	
	/* Compiles the information for Forward Primers into a single file */
	for (i = 0; cut[i] != '\0'; i = i+1)
	{
	
		temp[0] = '\0'; /* Resets the string */
		conc[0] = '\0';
		code_plus[0] = '\0';
		
		/* Every iteration replaces temp by a 1 base string, after which the string conc, containing the 3' end of the primer is added */
		for (x = 0; (T = Tm(code_plus)) < MAX_TM && x < 40 && oligo_cycles(code_plus) <= MAX_CYCLES && cut[i]-x >= 0 ; x++) { // The last criteria is to prevent non-existant code from being read
		
			temp[0] = code[cut[i]-x];
			temp[1] = '\0';
			strcat(temp, conc);
			strcpy(conc, temp);
			
			/* Used to check the next oligo stats */
			code_plus[0] = code[cut[i]-x-1];
			code_plus[1] = '\0';
			strcat(code_plus, conc);
								
		}
		
		complement(temp, noncoding);
		
		fprintf(output_file, "%d\t%d\t%f\t%d\t%d\t%s\t%s%s\t%s\t%s%s\t", i+1, cut[i] -1 , Tm(temp), oligo_cycles(temp), x, temp, F, temp, noncoding, noncoding,R);
		
		if (RepMask == 1) {
			
			extract(masked, cut[i]-x+1, cut[i], mask);
			complement(mask, ncmask);
			perc_repeat = ((double)counter(mask, 'N') / (double)x) * 100;
			
			fprintf(output_file, "%f\t%s\t%s", perc_repeat, mask, ncmask);
			//Print RepeatMasker Values
			
		}
		
		fprintf(output_file, "\n");
		
	}
			
	fclose(output_file);
	
	(void) time(&t2);
		
	printf("Results took %f minutes to be generated\n", (double) (t2 - t1) / 60 );
	printf("Exiting...\n\n");
	
	free(conc);
	free(code);
	if (RepMask ==1) {free(masked);}
	
	return EXIT_SUCCESS;
	
}
